import time

class Timer(object):
    """Timer class measures time needed to execute operations in a given context.
    Should measured time be needed, one can use time  module.

    - Example::

          from from lhcsmapi.Time import TimeTimer import Timer
          t = Timer()

          with Timer():
              for i in range(1000000):
                  i = i+1
          >>> Elapsed: 0.17786049842834473

          import time
          t = time.time()
          for i in range(1000000):
              i = i+1
          time.time() - t
          >>> 0.16668963432312012
    """

    def __enter__(self):
        self.tstart = time.time()

    def __exit__(self, type, value, traceback):
        print('Elapsed: {}'.format(time.time() - self.tstart))
