import pandas as pd
import numpy as np

class DetectBmodePattern(object):
    """DetectBmodePattern calculates timestamps corresponding to ramp-ups in the RB and RQ circuits from BMode variable"""

    @staticmethod
    def filter(bModeDf, bModeName, trigger, duration):
        """Function filters out ramps which are to short or incomplete"""
        triggerDf = pd.DataFrame(data=np.zeros(len(trigger)), index=trigger)
        i = 1
        toShort = 0
        incomplete = 0
        # only take dbsignal wich have all bmodes in the right order in it
        bModeSeries = bModeDf[bModeName]
        for b in range(4, len(bModeSeries)):
            if all([bModeSeries.iloc[b - 3 + t] == trigger[t] for t in range(len(trigger))]):
                if (bModeSeries.index[b - 2] - bModeSeries.index[b - 3] >= duration) and \
                        (bModeSeries.index[b] - bModeSeries.index[b - 1] >= duration):
                    newFrame = pd.DataFrame(bModeSeries.index[b - 3:b + 1], index=trigger)
                    newFrame.columns = [i]
                    i += 1
                    triggerDf = pd.concat([triggerDf, newFrame], axis=1, join='inner')
                else:
                    toShort += 1
            else:
                incomplete += 1

        incomplete = incomplete - (i - 2) * 4
        print(str(incomplete) + " triggers have been skiped because of incompleteness")
        print(str(toShort) + " ramps have not been taken because of to short up/down time")
        del triggerDf[0]

        return triggerDf
