import numpy as np

class SignalElectrical:
    """SignalElectrical class contains static methods for calculation of resistance from voltage and current dbsignal"""
    
    @staticmethod
    def calculateResistances(dataFrame, uNames, iNames, rNames):
        """Method calculates and appends resistance values to passed DataFrame based on the passed uNames and iNames"""
        # Following: https://stackoverflow.com/questions/26886653/pandas-create-new-column-based-on-values-from-other-columns
        for uName, iName, rName in zip(uNames, iNames, rNames):
            fn = lambda row: row[uName] / row[iName]  # define a function for the new column
            col = dataFrame.apply(fn, axis=1)  # get column data with an index
            dataFrame = dataFrame.assign(**{rName: col.values})  # assign values to column 'c'
        return dataFrame

    @staticmethod
    def filterResistancesForCurrentBelowThreshold(dataFrame, iNames, rNames, currentThresholdInAmps=1):
        """Method filters alle resistance values for current values below a certain value"""
        for iName, rName in zip(iNames, rNames):
            select_indices = list(np.where(abs(dataFrame[iName]) <= currentThresholdInAmps)[0])
            dataFrame[rName].iloc[select_indices] = 0  
    
 