from ..Time import Time as tc
import pandas as pd

class SignalUtilities(object):
    """SignalUtilities class contains static methods for processing dataframe indices"""
    
    @staticmethod
    def convertIndicesOfDataFrameOrSeries(dataStructure, conversionFunction, unit='ns', tz=None):
        """Method applies a conversion function to the index of the signal's DataFrame"""

        if isinstance(dataStructure, (pd.DataFrame, pd.Series)):
            if not dataStructure.empty:
                index = dataStructure.index.tolist()
                newIndex = [conversionFunction(t, unit, tz) for t in index]
                dataStructure.set_index([newIndex], inplace=True)
            return dataStructure
        else:
            raise TypeError("""Cannot convert the indices of input {}.
                            The data structure must be a pandas DataFrame or Series.""".format(type(dataStructure)))

    @staticmethod
    def convertIndicesToUnixTimestamp(dataStructure, unit='ns', tz=None):
        """Method coverts the indices of a pandas DataFrame to unix time (ns) and returns the DataFrame"""
        return SignalUtilities.convertIndicesOfDataFrameOrSeries(dataStructure, tc.convertToUnixTimestamp, unit, tz)
    
    @staticmethod
    def convertIndicesToSeconds(dataStructure, unit='ns', tz=None):
        """Method coverts the indices of a pandas DataFrame to unix time (s) and returns the DataFrame"""
        return SignalUtilities.convertIndicesOfDataFrameOrSeries(dataStructure, tc.convertToUnixTimeSeconds, unit, tz)

    @staticmethod
    def convertIndicesToTimestamps(dataStructure, unit='ns', tz=None):
        """Method coverts the indices of a pandas DataFrame to pandas Timestamps and returns the DataFrame"""
        return SignalUtilities.convertIndicesOfDataFrameOrSeries(dataStructure, tc.convertToPandasTimestamp, unit, tz)

    @staticmethod
    def synchronizeDataframe(df, t0=None):
        """Method coverts the indices of a pandas DataFrame to unix time (s) and synchronizes them then returns the DataFrame"""
        if t0 is None:
            t0 = df.index[0]

        newIndex = [idx - t0 for idx in df.index]
        df.set_index([newIndex], inplace=True)
        return df

