from ipywidgets import Layout, Box, Select
import functools

from lhcsmapi.gui.BaseModule import BaseModule
from lhcsmapi.gui.quench_heater.QhBrowserAction import QhBrowserAction
from lhcsmapi.metadata.SignalMetadata import SignalMetadata
from lhcsmapi.metadata.MappingMetadata import MappingMetadata


class QhBrowserBaseModule(BaseModule):
    class State(object):
        def __init__(self, widget):
            if widget.sel_cir.index is None:
                widget.sel_cir.index = 0
            self.cir = widget.sel_cir.options[widget.sel_cir.index]

            if widget.sel_cirname.index is None:
                widget.selmul_cirname.index = 0
            self.cirname = widget.sel_cirname.options[widget.sel_cirname.index]

            if widget.sel_qh.index is None:
                widget.sel_qh.index = 0
            self.qh = widget.sel_qh.options[widget.sel_qh.index]

            self.db = "PM"
            self.sys = "QH"

    def __init__(self):
        # Initialize widgets
        circuit_types = SignalMetadata.getCircuitTypes()
        circuit_types = ['RB']
        circuit_names = SignalMetadata.getCircuitNames(circuit_types[0])
        qh_per_circuit = MappingMetadata.getCellsForCircuitNames(circuit_types[0], circuit_names[0])

        # Single Selection
        # # Circuit
        self.sel_cir = Select(
            options=circuit_types, description='Type', index=0,
            layout=Layout(display="flex", flex_flow='column', width='15%')
        )
        self.sel_cir.observe(functools.partial(QhBrowserBaseModule.on_value_change_sel_cir, self=self), names='value')

        # Multiple Selection
        # # Circuit Name
        self.sel_cirname = Select(
            description='Circuit Name', options=circuit_names, index=0,
            layout=Layout(display="flex", flex_flow='column', width='15%')
        )
        self.sel_cirname.observe(functools.partial(QhBrowserBaseModule.on_value_change_sel_cirname, self=self),
                                 names='value')

        self.sel_qh = Select(
            description="Cell", options=qh_per_circuit, rows=15,
            layout=Layout(display="flex", flex_flow='column', width='15%')
        )
        self.sel_qh.observe(functools.partial(QhBrowserBaseModule.on_value_change_sel_qh, self=self), names='value')

        # Put together selection widgets
        self.widget = Box(children=[self.sel_cir, self.sel_cirname, self.sel_qh])

    def get_state(self):
        return QhBrowserBaseModule.State(self)

    @staticmethod
    def on_value_change_sel_cir(change, self):
        self.update_sel_cir(self.get_state())

    def update_sel_cir(self, state):
        with self.sel_cirname.hold_trait_notifications():
            self.sel_cirname.options = SignalMetadata.getCircuitNames(state.cir)
            self.sel_cirname.index = 0

    @staticmethod
    def on_value_change_sel_cirname(change, self):
        self.update_sel_qh(self.get_state())

    def update_sel_qh(self, state):
        with self.sel_qh.hold_trait_notifications():
            self.sel_qh.options = MappingMetadata.getCellsForCircuitNames(state.cir, state.cirname)

    @staticmethod
    def on_value_change_sel_qh(change, self):
        self.mediator.notify(self, QhBrowserAction.QH_NAME_CHANGED)