import unittest
import pandas as pd
from datetime import datetime
import pytz
from lhcsmapi.metadata.Metadata import Metadata


class TestMetadata(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

        # region convertToUnixTime

    def test_RB_Metadata_QPS_className(self):
        """Tests that Metadata returns correct className"""

        # arrange:
        circuitType = "RB"
        systemType = "QPS"
        component = "QH"
        database = "PM"

        className = "DQAMCNMB_PMHSU"

        # act:
        magnetTypeToSignalMetadata = Metadata.getMetadata(circuitType, systemType)

        # assert:
        self.assertEqual(className, magnetTypeToSignalMetadata[systemType][component][database]["className"])

    def test_RB_Metadata_QPS_system(self):
        """Tests that Metadata returns correct system"""

        # arrange:
        circuitType = "RB"
        systemType = "QPS"
        database = "PM"
        component = "QH"

        system = "QPS"

        # act:
        magnetTypeToSignalMetadata = Metadata.getMetadata(circuitType, systemType)

        # assert:
        self.assertEqual(system, magnetTypeToSignalMetadata[systemType][component][database]["system"])

    def test_RB_Metadata_QPS_source(self):
        """Tests that Metadata returns correct source"""

        # arrange:
        circuitType = "RB"
        systemType = "QPS"
        database = "PM"
        component = "QH"
        source = "*"

        # act:
        magnetTypeToSignalMetadata = Metadata.getMetadata(circuitType, systemType)

        # assert:
        self.assertEqual(source, magnetTypeToSignalMetadata[systemType][component][database]["source"])

    def test_RB_Metadata_QPS_PM_U_HDS(self):
        """Tests that Metadata returns correct U_HDS"""

        # arrange:
        circuitType = "RB"
        systemType = "QPS"
        database = "PM"
        component = "QH"

        U_HDS = ["U_HDS_1", "U_HDS_2", "U_HDS_3", "U_HDS_4"]

        # act:
        magnetTypeToSignalMetadata = Metadata.getMetadata(circuitType, systemType)

        # assert:
        self.assertEqual(U_HDS, magnetTypeToSignalMetadata[systemType][component][database]["U_HDS"])

    def test_RB_Metadata_QPS_PM_I_HDS(self):
        """Tests that Metadata returns correct source I_HDS"""

        # arrange:
        circuitType = "RB"
        systemType = "QPS"
        database = "PM"
        component = "QH"

        I_HDS = ["I_HDS_1", "I_HDS_2", "I_HDS_3", "I_HDS_4"]

        # act:
        magnetTypeToSignalMetadata = Metadata.getMetadata(circuitType, systemType)

        # assert:
        self.assertEqual(I_HDS, magnetTypeToSignalMetadata[systemType][component][database]["I_HDS"])

    def test_RB_Metadata_QPS_PM_FILTER_NAMES(self):
        """Tests that Metadata returns correct source FILTER_NAMES"""

        # arrange:
        circuitType = "RB"
        systemType = "QPS"
        database = "PM"
        component = "QH"


        FILTER_NAMES = ["U_HDS_1", "U_HDS_2", "U_HDS_3", "U_HDS_4", "I_HDS_1", "I_HDS_2", "I_HDS_3", "I_HDS_4"]
        # act:
        magnetTypeToSignalMetadata = Metadata.getMetadata(circuitType, systemType)

        # assert:
        self.assertEqual(FILTER_NAMES, magnetTypeToSignalMetadata[systemType][component][database]["FILTER_NAMES"])

    def test_RB_Metadata_QPS_R_HDS(self):
        """Tests that Metadata returns correct source R_HDS"""

        # arrange:
        circuitType = "RB"
        systemType = "QPS"
        component = "QH"

        R_HDS = ["R_HDS_1", "R_HDS_2", "R_HDS_3", "R_HDS_4"]
        # act:
        magnetTypeToSignalMetadata = Metadata.getMetadata(circuitType, systemType)

        # assert:
        self.assertEqual(R_HDS, magnetTypeToSignalMetadata[systemType][component]["R_HDS"])

    def test_RB_Metadata_QPS_dR_HDS(self):
        """Tests that Metadata returns correct source dR_HDS"""

        # arrange:
        circuitType = "RB"
        systemType = "QPS"
        component = "QH"

        dR_HDS = ["dR_HDS_1", "dR_HDS_2", "dR_HDS_3", "dR_HDS_4"]
        # act:
        magnetTypeToSignalMetadata = Metadata.getMetadata(circuitType, systemType)

        # assert:
        self.assertEqual(dR_HDS, magnetTypeToSignalMetadata[systemType][component]["dR_HDS"])

    def test_RB_Metadata_QPS_CALS_U_HDS(self):
        """Tests that Metadata returns correct U_HDS"""

        # arrange:
        circuitType = "RB"
        systemType = "QPS"
        component = "QH"
        database = "CALS"

        U_HDS = ["MB.{}:U_HDS_1", "MB.{}:U_HDS_2", "MB.{}:U_HDS_3", "MB.{}:U_HDS_4"]
        # act:
        magnetTypeToSignalMetadata = Metadata.getMetadata(circuitType, systemType)

        # assert:
        self.assertEqual(U_HDS, magnetTypeToSignalMetadata[systemType][component][database]["U_HDS"])

    def test_RB_Metadata_QPS_CALS_I_HDS(self):
        """Tests that Metadata returns correct I_HDS"""

        # arrange:
        circuitType = "RB"
        systemType = "QPS"
        component = "QH"
        database = "CALS"

        I_HDS = ["MB.{}:I_HDS_1", "MB.{}:I_HDS_2", "MB.{}:I_HDS_3", "MB.{}:I_HDS_4"]
        # act:
        magnetTypeToSignalMetadata = Metadata.getMetadata(circuitType, systemType)

        # assert:
        self.assertEqual(I_HDS, magnetTypeToSignalMetadata[systemType][component][database]["I_HDS"])

    def test_RB_Metadata_FGC_I_MEAS(self):
        """Tests that Metadata returns correct FGC I_MEAS"""

        # arrange:
        circuitType = "RB"
        systemType = "FGC"
        sector = "RB.A45"
        database = "CALS"
        variable = "I_MEAS"


        I_MEAS = "RPTE.UA47.RB.A45:I_MEAS"
        # act:
        metadata = Metadata.getMetadata(circuitType, sector)

        # assert:
        self.assertEqual(I_MEAS, metadata[sector][systemType][database][variable])

    def test_RB_Metadata_CircuitName(self):
        """Tests that Metadata returns correct I_MEAS"""

        # arrange:
        circuitType = "RB"
        sector = "RB.A12"
        variable = "PC_NAME"

        circuitNameRef = "RPTE.UA23.RB.A12"
        # act:
        metadata = Metadata.getMetadata(circuitType, sector)

        # assert:
        self.assertEqual(circuitNameRef, metadata[sector][variable])

    def test_RB_Metadata_PIC_ST_ABORT_PIC(self):
        """Tests that Metadata returns correct PIC ST_ABORT_PIC"""

        # arrange:
        circuitType = "RB"
        systemType = "PIC"
        sector = "RB.A12"
        variable = "ST_ABORT_PIC"
        database = "CALS"

        ST_ABORT_PIC = ["RB.A12.EVEN:ST_ABORT_PIC", "RB.A12.ODD:ST_ABORT_PIC"]
        # act:
        metadata = Metadata.getMetadata(circuitType, sector)

        # assert:
        self.assertEqual(ST_ABORT_PIC, metadata[sector][systemType][database][variable])

    def test_RB_Metadata_FGC_I_EARTH_PCNT(self):
        """Tests that Metadata returns correct FGC I_MEAS"""

        # arrange:
        circuitType = "RB"
        systemType = "FGC"
        sector = "RB.A12"
        variable = "I_EARTH_PCNT"
        database = "CALS"

        I_EARTH_PCNT = "RPTE.UA23.RB.A12:I_EARTH_PCNT"

        # act:
        metadata = Metadata.getMetadata(circuitType, sector)

        # assert:
        self.assertEqual(I_EARTH_PCNT, metadata[sector][systemType][database][variable])

    def test_RB_Metadata_getSectorNamesFromMagnetNames(self):
        """Tests that Metadata returns correct  sector name from magnet names"""

        # arrange
        circuitType = "RB"
        magnet = "B20L5"
        sector = "RB.A45"
        component = "MAGNET"

        # act
        sectorActual = Metadata.getSectorNamesFromComponents(circuitType, component, magnet)

        # assert
        self.assertEqual(sector, sectorActual)

    def test_RB_Metadata_getSectorNamesFromBusbarNames(self):
        """Tests that Metadata returns correct  sector name from busbar names"""

        # arrange
        circuitType = "RB"
        busbar = "DCBA.10L1.L:U_MAG"
        sector = "RB.A81"
        component = "BUSBAR"

        # act
        sectorActual = Metadata.getSectorNamesFromComponents(circuitType, component, busbar)

        # assert
        self.assertEqual(sector, sectorActual)

    def test_RB_Metadata_getSectorNamesFromCurrentName(self):
        """Tests that Metadata returns correct sector name from current name"""

        # arrange
        circuitType = "RB"
        current = "RPTE.UA23.RB.A12:I_MEAS"
        component = "FGC"

        sector = "RB.A12"

        # act
        sectorActual = Metadata.getSectorNamesFromComponents(circuitType, component, current)

        # assert
        self.assertEqual(sector, sectorActual)

    def test_RB_Metadata_getSectorNamesFromCurrentNames(self):
        """Tests that Metadata returns correct sector name from current names"""

        # arrange
        circuitType = "RB"
        current = ["RPTE.UA23.RB.A12:I_MEAS", "RPTE.UA27.RB.A23:I_MEAS"]
        component = "FGC"

        sector = ["RB.A12", "RB.A23"]

        # act
        sectorActual = Metadata.getSectorNamesFromComponents(circuitType, component, current)

        # assert
        self.assertEqual(sector, sectorActual)

    def test_RB_Metadata_getVoltageNamesRegExp(self):
        """Tests that Metadata returns correct sector name from current names"""

        # arrange
        circuitType = "RB"
        system = "QPS"
        component = "BUSBAR"
        database = "CALS"
        variable = "U_RES_REG_EXP"

        uResRegExpRef = "DCB%%:U_RES"

        # act
        uResRegExpActual = Metadata.getMetadata(circuitType, system)

        # assert
        self.assertEqual(uResRegExpRef, uResRegExpActual[system][component][database][variable])


    def test_RB_Metadata_getCurrentNamesRegExp(self):
        """Tests that Metadata returns correct current names regular expression"""

        # arrange
        circuitType = "RB"
        system = "FGC"
        database = "CALS"
        variable = "I_MEAS_REG_EXP"

        uResRegExpRef = "%.RB.A%:I_MEAS"

        # act
        uResRegExpActual = Metadata.getMetadata(circuitType, system)

        # assert
        self.assertEqual(uResRegExpRef, uResRegExpActual[system][database][variable])

    def test_RQD_Metadata_getSectorNamesFromBusbarNames(self):
        """Tests that Metadata returns correct  sector name from busbar names"""

        # arrange
        circuitType = "RQD"
        busbar = "DCBA.10L1.L:U_RES"
        sector = "RQD.A81"
        component = "BUSBAR"

        # act
        sectorActual = Metadata.getSectorNamesFromComponents(circuitType, component, busbar)

        # assert
        self.assertEqual(sector, sectorActual)

    def test_RQD_Metadata_getSectorNamesFromCurrentName(self):
        """Tests that Metadata returns correct sector name from current name"""

        # arrange
        circuitType = "RQD"
        current = "RPHE.UA63.RQD.A56:I_MEAS"
        component = "FGC"

        sector = "RQD.A56"

        # act
        sectorActual = Metadata.getSectorNamesFromComponents(circuitType, component, current)

        # assert
        self.assertEqual(sector, sectorActual)

    def test_RQF_Metadata_getSectorNamesFromBusbarNames(self):
        """Tests that Metadata returns correct  sector name from busbar names"""

        # arrange
        circuitType = "RQF"
        busbar = "DCQFQ.9L3.R:U_RES"
        sector = "RQF.A23"
        component = "BUSBAR"

        # act
        sectorActual = Metadata.getSectorNamesFromComponents(circuitType, component, busbar)

        # assert
        self.assertEqual(sector, sectorActual)

    def test_RQF_Metadata_getSectorNamesFromCurrentName(self):
        """Tests that Metadata returns correct sector name from current name"""

        # arrange
        circuitType = "RQF"
        current = "RPHE.UA23.RQF.A12:I_MEAS"
        component = "FGC"

        sector = "RQF.A12"

        # act
        sectorActual = Metadata.getSectorNamesFromComponents(circuitType, component, current)

        # assert
        self.assertEqual(sector, sectorActual)

    def test_getBeamModeDetails(self):
        """Tests that Metadata.getBeamModeDetails() returns correct name"""

        #arrange
        beamMode = 8
        name = "FLATTOP"
        description = "Flat top"

        # act
        beamModeDetails = Metadata.getBeamModeDetails()

        # assert
        self.assertEqual(beamModeDetails.query("Mode == 8")["Description"].values, description)

    def test_getBusbarToMagnetMapping_RB(self):
        """Tests that Metadata.getBusbarToMagnetMapping() returns correct name"""

        #arrange
        firstMagnet = "MB.A8L2"
        crate = "B8L2_1"

        # act
        busbarToMagnetMapping = Metadata.getBusbarToMagnetMapping("RB")

        # assert
        self.assertEqual(busbarToMagnetMapping[busbarToMagnetMapping['1st Magnet'].str.strip()==firstMagnet]["QPS Crate&Board"].values[0], crate)

    def test_getBusbarToMagnetMapping_RQD(self):
        """Tests that Metadata.getBusbarToMagnetMapping() returns correct name"""

        #arrange
        firstMagnet = "MQ.11L2.B2"
        crate = "B10L2_3"

        # act
        busbarToMagnetMapping = Metadata.getBusbarToMagnetMapping("RQD")

        # assert
        self.assertEqual(busbarToMagnetMapping[busbarToMagnetMapping['1st Magnet'].str.strip()==firstMagnet]["QPS Crate&Board"].values[0], crate)

    def test_getBusbarToMagnetMapping_RQF(self):
        """Tests that Metadata.getBusbarToMagnetMapping() returns correct name"""

        #arrange
        firstMagnet = "MQ.11L2.B2"
        crate = "B10L2_3"

        # act
        busbarToMagnetMapping = Metadata.getBusbarToMagnetMapping("RQF")

        # assert
        self.assertEqual(busbarToMagnetMapping[busbarToMagnetMapping['1st Magnet'].str.strip()==firstMagnet]["QPS Crate&Board"].values[0], crate)

    def test_getCrateToMagnetAndVoltageFeelerMapping_RB(self):
        """Tests that Metadata.getCrateToMagnetAndVoltageFeelerMapping() returns correct name"""

        #arrange
        magnet = "MB.A25R1"
        crate = "B25R1"

        # act
        crateToMagnetandVf = Metadata.getCrateToMagnetAndVoltageFeelerMapping()

        # assert
        self.assertEqual(crateToMagnetandVf[crateToMagnetandVf['Magnet2'].str.strip()==magnet]["Crate"].values[0], crate)

    def test_getCrateNameFromMagnetName_RB_A25R1(self):
        """Tests that Metadata.getCrateToMagnetAndVoltageFeelerMapping() returns correct name"""

        #arrange
        magnet = "MB.A25R1"
        crate = "B25R1"

        # act
        crateAct = Metadata.getCrateNameFromMagnetName(magnet)

        # assert
        self.assertEqual(crate, crateAct)

    def test_getCrateNameFromMagnetName_RB_A30L2(self):
        """Tests that Metadata.getCrateToMagnetAndVoltageFeelerMapping() returns correct name"""

        #arrange
        magnet = "MB.A30L2"
        crate = "B30L2"

        # act
        crateAct = Metadata.getCrateNameFromMagnetName(magnet)

        # assert
        self.assertEqual(crate, crateAct)

    def test_getMagnetNamesFromCrateName_RB(self):
        """Tests that Metadata.getCrateToMagnetAndVoltageFeelerMapping() returns correct name"""

        #arrange
        crate = "B25R1"
        magnets = ['MB.B24R1', 'MB.A25R1', 'MB.C25R1']


        # act
        magnetsAct = Metadata.getMagnetNamesFromCrateName(crate)

        # assert
        self.assertEqual(magnets, magnetsAct)

