import unittest
from lhcsmapi.metadata.SignalMetadata import SignalMetadata


class TestSignalMetadata(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_getCircuitTypes(self):
        # arrange

        # act
        circuitTypesAct = SignalMetadata.getCircuitTypes()

        # assert
        circuitTypesExp = ['IPD2', 'IPD2_B1B2', 'IPQ2', 'IPQ4', 'IPQ8', 'RB', 'RQD', 'RQF', '600A']
        circuitTypesAct.sort()
        circuitTypesExp.sort()
        self.assertEqual(circuitTypesExp, circuitTypesAct)

    def test_getCircuitNames(self):
        # arrange
        circuitType = "RB"

        # act
        circuitNamesAct = SignalMetadata.getCircuitNames(circuitType)

        # assert
        circuitNamesExp = ["RB.A12", "RB.A23", "RB.A34", "RB.A45", "RB.A56", "RB.A67", "RB.A78", "RB.A81"]
        circuitNamesAct.sort()
        circuitNamesExp.sort()
        self.assertEqual(circuitNamesExp, circuitNamesAct)

    def test_getVariableTypesPerCircuit(self):
        # arrange
        circuitType = "RB"

        # act
        systemTypesPerCircuitAct = SignalMetadata.getSystemTypesPerCircuit(circuitType)

        # assert
        systemTypesPerCircuitExp = ['CIP', 'CRYO', 'PIC', 'PC', 'QDS', 'QH', 'BUSBAR', 'DIODE_RB', 'VF', 'LEADS_EVEN',
                                    'LEADS_ODD', 'EE']
        systemTypesPerCircuitAct.sort()
        systemTypesPerCircuitExp.sort()
        self.assertEqual(systemTypesPerCircuitExp, systemTypesPerCircuitAct)

    def test_getVariableNames_CALS(self):
        # arrange
        circuitType = "RB"
        circuitName = "RB.A56"
        systemType = "PC"
        databaseName = "CALS"

        # act
        variableNamesAct = SignalMetadata.getVariableNames(circuitType, circuitName, systemType, databaseName)

        # assert
        variableNamesExp = ['I_REF', 'I_MEAS', 'V_MEAS', 'V_REF', 'I_EARTH', 'I_EARTH_PCNT']
        variableNamesAct.sort()
        variableNamesExp.sort()
        self.assertEqual(variableNamesExp, variableNamesAct)

    def test_getVariableNames_CALS_VariableTypeKeyError(self):
        # arrange
        circuitType = "RB"
        circuitName = "RB.A34"
        systemType = "PCX"
        databaseName = "CALS"

        # act

        # assert
        with self.assertRaises(KeyError):
            SignalMetadata.getVariableNames(circuitType, circuitName, systemType, databaseName)

    def test_getVariableNames_CALS_DatabaseNameKeyError(self):
        # arrange
        circuitType = "RB"
        circuitName = "RB.A45"
        systemType = "PC"
        databaseName = "NXCALS"

        # act

        # assert
        with self.assertRaises(KeyError):
            SignalMetadata.getVariableNames(circuitType, circuitName, systemType, databaseName)

    def test_getVariableNames_PM(self):
        # arrange
        circuitType = "RB"
        circuitName = "RB.A12"
        systemType = "PC"
        databaseName = "PM"

        # act
        variableNamesAct = SignalMetadata.getVariableNames(circuitType, circuitName, systemType, databaseName)

        # assert
        variableNamesExp = ['V_REF', 'I_MEAS', 'I_EARTH', 'V_MEAS', 'I_EARTH_PCNT', 'I_A', 'I_REF', 'I_B']
        variableNamesAct.sort()
        variableNamesExp.sort()
        self.assertEqual(variableNamesExp, variableNamesAct)

    def test_getVariableName_PC_I_MEAS(self):
        # arrange
        circuitType = "RB"
        circuitName = "RB.A12"
        systemType = "PC"
        databaseName = "CALS"
        variableName = "I_MEAS"

        # act
        variableNameAct = SignalMetadata.getVariableName(circuitType, circuitName, systemType, databaseName, variableName)

        # assert
        variableNameExp = "RPTE.UA23.RB.A12:I_MEAS"
        self.assertEqual(variableNameExp, variableNameAct)

    def test_getVariableName_QH_I_HDS(self):
        # arrange
        circuitType = "RB"
        circuitName = "RB.A12"
        systemType = "QH"
        databaseName = "CALS"
        variableName = "I_HDS"

        # act
        variableNameAct = SignalMetadata.getVariableName(circuitType, circuitName, systemType, databaseName, variableName)

        # assert
        variableNameExp = ["%MAGNET%:I_HDS_1", "%MAGNET%:I_HDS_2", "%MAGNET%:I_HDS_3", "%MAGNET%:I_HDS_4"]
        self.assertEqual(variableNameExp, variableNameAct)

    def test_getVariableName_PM_QH_I_HDS_METADATA(self):
        # arrange
        circuitType = "RB"
        circuitName = "RB.A12"
        systemType = "QH"
        databaseName = "PM"

        # act
        metadata = SignalMetadata.getCircuitVariableDatabaseMetadata(circuitType, circuitName, systemType, databaseName)
        systemAct, classNameAct, sourceAct = metadata['system'], metadata['className'], metadata['source']

        # assert
        sourceExp = "%CELL%"
        classNameExp = "DQAMCNMB_PMHSU"
        systemExp = "QPS"
        self.assertEqual(sourceExp, sourceAct)
        self.assertEqual(classNameExp, classNameAct)
        self.assertEqual(systemExp, systemAct)

    def test_getVariableName_PM_PC__METADATA(self):
        # arrange
        circuitType = "RB"
        systemType = "PC"
        circuitName = "RB.A12"
        databaseName = "PM"

        # act
        metadata = SignalMetadata.getCircuitVariableDatabaseMetadata(circuitType, circuitName, systemType, databaseName)
        systemAct, classNameAct, sourceAct = metadata['system'], metadata['className'], metadata['source']

        # assert
        sourceExp = "RPTE.UA23.RB.A12"
        classNameExp = "51_self_pmd"
        systemExp = "FGC"
        self.assertEqual(sourceExp, sourceAct)
        self.assertEqual(classNameExp, classNameAct)
        self.assertEqual(systemExp, systemAct)

    def test_getVariableName_PM_LEADS_EVEN_METADATA(self):
        # arrange
        circuitType = "RB"
        systemType = "LEADS_EVEN"
        circuitName = "RB.A34"
        databaseName = "PM"

        # act
        metadata = SignalMetadata.getCircuitVariableDatabaseMetadata(circuitType, circuitName, systemType, databaseName)
        systemAct, classNameAct, sourceAct = metadata['system'], metadata['className'], metadata['source']

        # assert
        sourceExp = "RB.A34"
        classNameExp = "DQAMGNDRBEVEN"
        systemExp = "QPS"
        self.assertEqual(sourceExp, sourceAct)
        self.assertEqual(classNameExp, classNameAct)
        self.assertEqual(systemExp, systemAct)

    def test_getVariableName_PM_LEADS_ODD_METADATA(self):
        # arrange
        circuitType = "RB"
        systemType = "LEADS_ODD"
        circuitName = "RB.A45"
        databaseName = "PM"

        # act
        metadata = SignalMetadata.getCircuitVariableDatabaseMetadata(circuitType, circuitName, systemType, databaseName)
        systemAct, classNameAct, sourceAct = metadata['system'], metadata['className'], metadata['source']

        # assert
        sourceExp = "RB.A45"
        classNameExp = "DQAMGNDRBODD"
        systemExp = "QPS"
        self.assertEqual(sourceExp, sourceAct)
        self.assertEqual(classNameExp, classNameAct)
        self.assertEqual(systemExp, systemAct)

    def test_getVariableName_PM_EE_METADATA(self):
        # arrange
        circuitType = "RB"
        systemType = "EE"
        circuitName = "RB.A23"
        databaseName = "PM"

        # act
        metadata = SignalMetadata.getCircuitVariableDatabaseMetadata(circuitType, circuitName, systemType, databaseName)
        systemAct, classNameAct, sourceAct = metadata['system'], metadata['className'], metadata['source']

        # assert
        sourceExp = ["RR17.RB.A12", "UA23.RB.A12"]
        classNameExp = "DQAMSNRB"
        systemExp = "QPS"

        sourceAct.sort()
        sourceExp.sort()
        self.assertEqual(sourceExp, sourceAct)
        self.assertEqual(classNameExp, classNameAct)
        self.assertEqual(systemExp, systemAct)

    def test_getVariableName_CALS_PIC_ST_ABORT_PIC(self):
        # arrange
        circuitType = "RB"
        circuitName = "RB.A12"
        systemType = "PIC"
        databaseName = "CALS"
        variableName = "ST_ABORT_PIC"

        # act
        variableNameAct = SignalMetadata.getVariableName(circuitType, circuitName, systemType, databaseName, variableName)

        # assert
        variableNameExp = ["RB.A12.ODD:ST_ABORT_PIC", "RB.A12.EVEN:ST_ABORT_PIC"]
        variableNameAct.sort()
        variableNameExp.sort()
        self.assertEqual(variableNameExp, variableNameAct)

    def test_getVariableNames_PM_PC_I_MEAS_I_A_I_REF(self):
        # arrange
        circuitType = "RB"
        circuitName = "RB.A12"
        systemType = "PC"
        databaseName = "PM"
        variableNames = ["I_REF", "I_MEAS", "V_REF", "V_MEAS", "I_EARTH", "I_EARTH_PCNT", "I_A", "I_B"]

        # act
        variableNameAct = SignalMetadata.getVariableName(circuitType, circuitName, systemType, databaseName, variableNames)

        # assert
        variableNameExp = ["STATUS.I_REF", "STATUS.I_MEAS", "STATUS.V_REF", "STATUS.V_MEAS", "IEARTH.IEARTH",
                           "STATUS.I_EARTH_PCNT", "IAB.I_A", "IAB.I_B"]
        variableNameAct.sort()
        variableNameExp.sort()
        self.assertEqual(variableNameExp, variableNameAct)

    def test_getVariableNames_PM_QDS_U_QS0_U_1_U_2(self):
        # arrange
        circuitType = "RB"
        circuitName = "RB.A12"
        systemType = "QDS"
        databaseName = "PM"
        variableNames = ['U_QS0', 'U_1', 'U_2']

        # act
        variableNameAct = SignalMetadata.getVariableName(circuitType, circuitName, systemType, databaseName, variableNames)

        # assert
        variableNameExp = ['%CELL%:U_1', '%CELL%:U_2', '%CELL%:U_QS0']
        variableNameAct.sort()
        variableNameExp.sort()
        self.assertEqual(variableNameExp, variableNameAct)

    def test_getVariableName_CALS_LEADS_ODD_U_BB_2(self):
        # arrange
        circuitType = "RB"
        circuitName = "RB.A12"
        systemType = "LEADS_ODD"
        databaseName = "CALS"
        variableName = "U_BB_2"

        # act
        variableNameAct = SignalMetadata.getVariableName(circuitType, circuitName, systemType, databaseName, variableName)

        # assert
        variableNameExp = "RB.A12:U_BB_2"
        self.assertEqual(variableNameExp, variableNameAct)

    def test_getVariableName_CALS_LEADS_ODD_U_BB_2(self):
        # arrange
        circuitType = "RB"
        circuitNames = ["RB.A12", "RB.A23"]
        systemType = "LEADS_ODD"
        databaseName = "CALS"
        variableName = "U_BB_2"

        # act
        variableNameAct = SignalMetadata.getVariableName(circuitType, circuitNames, systemType, databaseName, variableName)

        # assert
        variableNameExp = ["RB.A12:U_BB_2", "RB.A23:U_BB_2"]
        variableNameAct.sort()
        variableNameExp.sort()
        self.assertEqual(variableNameExp, variableNameAct)

    def test_getVariableName_CALS_BUSBAR_U_RES(self):
        # arrange
        circuitType = "RB"
        circuitName = "RB.A12"
        systemType = "BUSBAR"
        databaseName = "CALS"
        variableName = "U_RES"

        # act
        variableNameAct = SignalMetadata.getVariableName(circuitType, circuitName, systemType, databaseName, variableName)

        # assert
        variableNameExp = "%BUSBAR%:U_RES"
        self.assertEqual(variableNameExp, variableNameAct)

    def test_getDatabaseNames_LEADS_ODD(self):
        # arrange
        circuitType = "RB"
        systemType = "LEADS_ODD"

        # act
        databaseNamesAct = SignalMetadata.getDatabaseNames(circuitType, systemType)

        # assert
        databaseNamesExp = ['PM', 'CALS']
        databaseNamesAct.sort()
        databaseNamesExp.sort()
        self.assertEqual(databaseNamesExp, databaseNamesAct)

    def test_getDatabaseNames_PIC(self):
        # arrange
        circuitType = "RB"
        systemType = "PIC"

        # act
        databaseNamesAct = SignalMetadata.getDatabaseNames(circuitType, systemType)

        # assert
        databaseNamesExp = ['CALS']
        databaseNamesAct.sort()
        databaseNamesExp.sort()
        self.assertEqual(databaseNamesExp, databaseNamesAct)

    def test_getCircuitNameFromMagnetOrCrate(self):
        # arrange
        circuitType = 'RB'
        magnetName = "A30L2"

        # act
        circuitNameAct = SignalMetadata.getCircuitNameFromMagnetOrCrate(circuitType, magnetName)

        # assert
        circuitNameExp = 'RB.A12'
        self.assertEqual(circuitNameExp, circuitNameAct)

    def test_getWildcardForCircuitSystemDbSignal_RB_RBA12_QH_PM_U_HDS(self):
        # arrange
        circuit = 'RB'
        circuitName = 'RB.A12'
        system = 'QH'
        db = 'PM'
        signal = 'U_HDS'

        # act
        wildcardAct = SignalMetadata.getWildcardForCircuitSystemDbSignal(circuit, circuitName, system, db, signal)
        wildcardNameAct = wildcardAct[0]
        wildcardValuesAct = wildcardAct[1]

        # assert
        wildcardNameExp = 'Cell'
        wildcardValuesExp = ['A8R1', 'B8R1', 'A9R1', 'B9R1', 'A10R1', 'B10R1', 'A11R1', 'B11R1', 'A12R1', 'B12R1',
                             'C12R1', 'A13R1', 'B13R1', 'C13R1', 'A14R1', 'B14R1', 'C14R1', 'A15R1', 'B15R1', 'C15R1',
                             'A16R1', 'B16R1', 'C16R1', 'A17R1', 'B17R1', 'C17R1', 'A18R1', 'B18R1', 'C18R1', 'A19R1',
                             'B19R1', 'C19R1', 'A20R1', 'B20R1', 'C20R1', 'A21R1', 'B21R1', 'C21R1', 'A22R1', 'B22R1',
                             'C22R1', 'A23R1', 'B23R1', 'C23R1', 'A24R1', 'B24R1', 'C24R1', 'A25R1', 'B25R1', 'C25R1',
                             'A26R1', 'B26R1', 'C26R1', 'A27R1', 'B27R1', 'C27R1', 'A28R1', 'B28R1', 'C28R1', 'A29R1',
                             'B29R1', 'C29R1', 'A30R1', 'B30R1', 'C30R1', 'A31R1', 'B31R1', 'C31R1', 'A32R1', 'B32R1',
                             'C32R1', 'A33R1', 'B33R1', 'C33R1', 'A34R1', 'B34R1', 'C34R1', 'C34L2', 'B34L2', 'A34L2',
                             'C33L2', 'B33L2', 'A33L2', 'C32L2', 'B32L2', 'A32L2', 'C31L2', 'B31L2', 'A31L2', 'C30L2',
                             'B30L2', 'A30L2', 'C29L2', 'B29L2', 'A29L2', 'C28L2', 'B28L2', 'A28L2', 'C27L2', 'B27L2',
                             'A27L2', 'C26L2', 'B26L2', 'A26L2', 'C25L2', 'B25L2', 'A25L2', 'C24L2', 'B24L2', 'A24L2',
                             'C23L2', 'B23L2', 'A23L2', 'C22L2', 'B22L2', 'A22L2', 'C21L2', 'B21L2', 'A21L2', 'C20L2',
                             'B20L2', 'A20L2', 'C19L2', 'B19L2', 'A19L2', 'C18L2', 'B18L2', 'A18L2', 'C17L2', 'B17L2',
                             'A17L2', 'C16L2', 'B16L2', 'A16L2', 'C15L2', 'B15L2', 'A15L2', 'C14L2', 'B14L2', 'A14L2',
                             'C13L2', 'B13L2', 'A13L2', 'C12L2', 'B12L2', 'A12L2', 'B11L2', 'A11L2', 'B10L2', 'A10L2',
                             'B9L2', 'A9L2', 'B8L2', 'A8L2']

        self.assertEqual(wildcardNameExp, wildcardNameAct)
        wildcardValuesExp.sort()
        wildcardValuesAct.sort()
        self.assertEqual(wildcardValuesExp, wildcardValuesAct)

    def test_getWildcardForCircuitSystemDbSignal_RB_RBA12_QH_CALS_U_HDS(self):
        # arrange
        circuit = 'RB'
        circuitName = 'RB.A12'
        system = 'QH'
        db = 'CALS'
        signal = 'U_HDS'

        # act
        wildcardAct = SignalMetadata.getWildcardForCircuitSystemDbSignal(circuit, circuitName, system, db, signal)
        wildcardNameAct = wildcardAct[0]
        wildcardValuesAct = wildcardAct[1]

        # assert
        wildcardNameExp = 'Magnet'
        wildcardValuesExp = ['MB.A8R1', 'MB.B8R1', 'MB.A9R1', 'MB.B9R1', 'MB.A10R1', 'MB.B10R1', 'MB.A11R1', 'MB.B11R1', 'MB.A12R1', 'MB.B12R1',
                             'MB.C12R1', 'MB.A13R1', 'MB.B13R1', 'MB.C13R1', 'MB.A14R1', 'MB.B14R1', 'MB.C14R1', 'MB.A15R1', 'MB.B15R1', 'MB.C15R1',
                             'MB.A16R1', 'MB.B16R1', 'MB.C16R1', 'MB.A17R1', 'MB.B17R1', 'MB.C17R1', 'MB.A18R1', 'MB.B18R1', 'MB.C18R1', 'MB.A19R1',
                             'MB.B19R1', 'MB.C19R1', 'MB.A20R1', 'MB.B20R1', 'MB.C20R1', 'MB.A21R1', 'MB.B21R1', 'MB.C21R1', 'MB.A22R1', 'MB.B22R1',
                             'MB.C22R1', 'MB.A23R1', 'MB.B23R1', 'MB.C23R1', 'MB.A24R1', 'MB.B24R1', 'MB.C24R1', 'MB.A25R1', 'MB.B25R1', 'MB.C25R1',
                             'MB.A26R1', 'MB.B26R1', 'MB.C26R1', 'MB.A27R1', 'MB.B27R1', 'MB.C27R1', 'MB.A28R1', 'MB.B28R1', 'MB.C28R1', 'MB.A29R1',
                             'MB.B29R1', 'MB.C29R1', 'MB.A30R1', 'MB.B30R1', 'MB.C30R1', 'MB.A31R1', 'MB.B31R1', 'MB.C31R1', 'MB.A32R1', 'MB.B32R1',
                             'MB.C32R1', 'MB.A33R1', 'MB.B33R1', 'MB.C33R1', 'MB.A34R1', 'MB.B34R1', 'MB.C34R1', 'MB.C34L2', 'MB.B34L2', 'MB.A34L2',
                             'MB.C33L2', 'MB.B33L2', 'MB.A33L2', 'MB.C32L2', 'MB.B32L2', 'MB.A32L2', 'MB.C31L2', 'MB.B31L2', 'MB.A31L2', 'MB.C30L2',
                             'MB.B30L2', 'MB.A30L2', 'MB.C29L2', 'MB.B29L2', 'MB.A29L2', 'MB.C28L2', 'MB.B28L2', 'MB.A28L2', 'MB.C27L2', 'MB.B27L2',
                             'MB.A27L2', 'MB.C26L2', 'MB.B26L2', 'MB.A26L2', 'MB.C25L2', 'MB.B25L2', 'MB.A25L2', 'MB.C24L2', 'MB.B24L2', 'MB.A24L2',
                             'MB.C23L2', 'MB.B23L2', 'MB.A23L2', 'MB.C22L2', 'MB.B22L2', 'MB.A22L2', 'MB.C21L2', 'MB.B21L2', 'MB.A21L2', 'MB.C20L2',
                             'MB.B20L2', 'MB.A20L2', 'MB.C19L2', 'MB.B19L2', 'MB.A19L2', 'MB.C18L2', 'MB.B18L2', 'MB.A18L2', 'MB.C17L2', 'MB.B17L2',
                             'MB.A17L2', 'MB.C16L2', 'MB.B16L2', 'MB.A16L2', 'MB.C15L2', 'MB.B15L2', 'MB.A15L2', 'MB.C14L2', 'MB.B14L2', 'MB.A14L2',
                             'MB.C13L2', 'MB.B13L2', 'MB.A13L2', 'MB.C12L2', 'MB.B12L2', 'MB.A12L2', 'MB.B11L2', 'MB.A11L2', 'MB.B10L2', 'MB.A10L2',
                             'MB.B9L2', 'MB.A9L2', 'MB.B8L2', 'MB.A8L2']

        self.assertEqual(wildcardNameExp, wildcardNameAct)
        wildcardValuesExp.sort()
        wildcardValuesAct.sort()
        self.assertEqual(wildcardValuesExp, wildcardValuesAct)

    def test_getWildcardForCircuitSystemDbSignal_RB_RBA12_QDS_PM_U_HDS(self):
        # arrange
        circuit = 'RB'
        circuitName = 'RB.A12'
        system = 'QDS'
        db = 'PM'
        signal = 'U_HDS'

        # act
        wildcardAct = SignalMetadata.getWildcardForCircuitSystemDbSignal(circuit, circuitName, system, db, signal)
        wildcardNameAct = wildcardAct[0]
        wildcardValuesAct = wildcardAct[1]

        # assert
        wildcardNameExp = 'Cell'
        wildcardValuesExp = ['A8R1', 'B8R1', 'A9R1', 'B9R1', 'A10R1', 'B10R1', 'A11R1', 'B11R1', 'A12R1', 'B12R1',
                             'C12R1', 'A13R1', 'B13R1', 'C13R1', 'A14R1', 'B14R1', 'C14R1', 'A15R1', 'B15R1', 'C15R1',
                             'A16R1', 'B16R1', 'C16R1', 'A17R1', 'B17R1', 'C17R1', 'A18R1', 'B18R1', 'C18R1', 'A19R1',
                             'B19R1', 'C19R1', 'A20R1', 'B20R1', 'C20R1', 'A21R1', 'B21R1', 'C21R1', 'A22R1', 'B22R1',
                             'C22R1', 'A23R1', 'B23R1', 'C23R1', 'A24R1', 'B24R1', 'C24R1', 'A25R1', 'B25R1', 'C25R1',
                             'A26R1', 'B26R1', 'C26R1', 'A27R1', 'B27R1', 'C27R1', 'A28R1', 'B28R1', 'C28R1', 'A29R1',
                             'B29R1', 'C29R1', 'A30R1', 'B30R1', 'C30R1', 'A31R1', 'B31R1', 'C31R1', 'A32R1', 'B32R1',
                             'C32R1', 'A33R1', 'B33R1', 'C33R1', 'A34R1', 'B34R1', 'C34R1', 'C34L2', 'B34L2', 'A34L2',
                             'C33L2', 'B33L2', 'A33L2', 'C32L2', 'B32L2', 'A32L2', 'C31L2', 'B31L2', 'A31L2', 'C30L2',
                             'B30L2', 'A30L2', 'C29L2', 'B29L2', 'A29L2', 'C28L2', 'B28L2', 'A28L2', 'C27L2', 'B27L2',
                             'A27L2', 'C26L2', 'B26L2', 'A26L2', 'C25L2', 'B25L2', 'A25L2', 'C24L2', 'B24L2', 'A24L2',
                             'C23L2', 'B23L2', 'A23L2', 'C22L2', 'B22L2', 'A22L2', 'C21L2', 'B21L2', 'A21L2', 'C20L2',
                             'B20L2', 'A20L2', 'C19L2', 'B19L2', 'A19L2', 'C18L2', 'B18L2', 'A18L2', 'C17L2', 'B17L2',
                             'A17L2', 'C16L2', 'B16L2', 'A16L2', 'C15L2', 'B15L2', 'A15L2', 'C14L2', 'B14L2', 'A14L2',
                             'C13L2', 'B13L2', 'A13L2', 'C12L2', 'B12L2', 'A12L2', 'B11L2', 'A11L2', 'B10L2', 'A10L2',
                             'B9L2', 'A9L2', 'B8L2', 'A8L2']

        self.assertEqual(wildcardNameExp, wildcardNameAct)
        wildcardValuesExp.sort()
        wildcardValuesAct.sort()
        self.assertEqual(wildcardValuesExp, wildcardValuesAct)

    def test_getWildcardForCircuitSystemDbSignal_RB_RBA12_QDS_CALS_U_HDS(self):
        # arrange
        circuit = 'RB'
        circuitName = 'RB.A12'
        system = 'QDS'
        db = 'CALS'
        signal = 'U_HDS'

        # act
        wildcardAct = SignalMetadata.getWildcardForCircuitSystemDbSignal(circuit, circuitName, system, db, signal)
        wildcardNameAct = wildcardAct[0]
        wildcardValuesAct = wildcardAct[1]

        # assert
        wildcardNameExp = 'Magnet'
        wildcardValuesExp = ['MB.A8R1', 'MB.B8R1', 'MB.A9R1', 'MB.B9R1', 'MB.A10R1', 'MB.B10R1', 'MB.A11R1', 'MB.B11R1', 'MB.A12R1', 'MB.B12R1',
                             'MB.C12R1', 'MB.A13R1', 'MB.B13R1', 'MB.C13R1', 'MB.A14R1', 'MB.B14R1', 'MB.C14R1', 'MB.A15R1', 'MB.B15R1', 'MB.C15R1',
                             'MB.A16R1', 'MB.B16R1', 'MB.C16R1', 'MB.A17R1', 'MB.B17R1', 'MB.C17R1', 'MB.A18R1', 'MB.B18R1', 'MB.C18R1', 'MB.A19R1',
                             'MB.B19R1', 'MB.C19R1', 'MB.A20R1', 'MB.B20R1', 'MB.C20R1', 'MB.A21R1', 'MB.B21R1', 'MB.C21R1', 'MB.A22R1', 'MB.B22R1',
                             'MB.C22R1', 'MB.A23R1', 'MB.B23R1', 'MB.C23R1', 'MB.A24R1', 'MB.B24R1', 'MB.C24R1', 'MB.A25R1', 'MB.B25R1', 'MB.C25R1',
                             'MB.A26R1', 'MB.B26R1', 'MB.C26R1', 'MB.A27R1', 'MB.B27R1', 'MB.C27R1', 'MB.A28R1', 'MB.B28R1', 'MB.C28R1', 'MB.A29R1',
                             'MB.B29R1', 'MB.C29R1', 'MB.A30R1', 'MB.B30R1', 'MB.C30R1', 'MB.A31R1', 'MB.B31R1', 'MB.C31R1', 'MB.A32R1', 'MB.B32R1',
                             'MB.C32R1', 'MB.A33R1', 'MB.B33R1', 'MB.C33R1', 'MB.A34R1', 'MB.B34R1', 'MB.C34R1', 'MB.C34L2', 'MB.B34L2', 'MB.A34L2',
                             'MB.C33L2', 'MB.B33L2', 'MB.A33L2', 'MB.C32L2', 'MB.B32L2', 'MB.A32L2', 'MB.C31L2', 'MB.B31L2', 'MB.A31L2', 'MB.C30L2',
                             'MB.B30L2', 'MB.A30L2', 'MB.C29L2', 'MB.B29L2', 'MB.A29L2', 'MB.C28L2', 'MB.B28L2', 'MB.A28L2', 'MB.C27L2', 'MB.B27L2',
                             'MB.A27L2', 'MB.C26L2', 'MB.B26L2', 'MB.A26L2', 'MB.C25L2', 'MB.B25L2', 'MB.A25L2', 'MB.C24L2', 'MB.B24L2', 'MB.A24L2',
                             'MB.C23L2', 'MB.B23L2', 'MB.A23L2', 'MB.C22L2', 'MB.B22L2', 'MB.A22L2', 'MB.C21L2', 'MB.B21L2', 'MB.A21L2', 'MB.C20L2',
                             'MB.B20L2', 'MB.A20L2', 'MB.C19L2', 'MB.B19L2', 'MB.A19L2', 'MB.C18L2', 'MB.B18L2', 'MB.A18L2', 'MB.C17L2', 'MB.B17L2',
                             'MB.A17L2', 'MB.C16L2', 'MB.B16L2', 'MB.A16L2', 'MB.C15L2', 'MB.B15L2', 'MB.A15L2', 'MB.C14L2', 'MB.B14L2', 'MB.A14L2',
                             'MB.C13L2', 'MB.B13L2', 'MB.A13L2', 'MB.C12L2', 'MB.B12L2', 'MB.A12L2', 'MB.B11L2', 'MB.A11L2', 'MB.B10L2', 'MB.A10L2',
                             'MB.B9L2', 'MB.A9L2', 'MB.B8L2', 'MB.A8L2']

        self.assertEqual(wildcardNameExp, wildcardNameAct)
        wildcardValuesExp.sort()
        wildcardValuesAct.sort()
        self.assertEqual(wildcardValuesExp, wildcardValuesAct)

    def test_getWildcardForCircuitSystemDbSignal_RQ_RQDA12_QH_CALS_U_HDS(self):
        # arrange
        circuit = 'RQ'
        circuitName = 'RQD.A12'
        system = 'QH'
        db = 'CALS'
        signal = 'U_HDS'

        # act
        wildcardAct = SignalMetadata.getWildcardForCircuitSystemDbSignal(circuit, circuitName, system, db, signal)

        # assert
        wildcardExp = []
        self.assertEqual(wildcardExp, wildcardAct)

    def test_getWildcardForCircuitSystemDbSignal_RB_RBA12_QH_NXCALS_U_HDS(self):
        # arrange
        circuit = 'RB'
        circuitName = 'RB.A12'
        system = 'QH'
        db = 'NXCALS'
        signal = 'U_HDS'

        # act

        # assert
        with self.assertRaises(KeyError):
            SignalMetadata.getWildcardForCircuitSystemDbSignal(circuit, circuitName, system, db, signal)

    def test_getWildcardForCircuitSystemDbSignal_RB_RBA12_VF_PM_U_EARTH(self):
        # arrange
        circuit = 'RB'
        circuitName = 'RB.A12'
        system = 'VF'
        db = 'PM'
        signal = 'U_EARTH'

        # act
        wildcardAct = SignalMetadata.getWildcardForCircuitSystemDbSignal(circuit, circuitName, system, db, signal)
        wildcardNameAct = wildcardAct[0]
        wildcardValuesAct = wildcardAct[1]

        # assert
        wildcardNameExp = 'Voltage feeler'
        wildcardValuesExp = ['A8L2', 'A10L2', 'C12L2', 'C14L2', 'C16L2', 'C18L2', 'C20L2', 'C22L2', 'C24L2', 'C26L2',
                             'C28L2', 'C30L2', 'C32L2', 'C34L2', 'A33R1', 'C31R1', 'C29R1', 'C27R1', 'C25R1', 'C23R1',
                             'C21R1', 'C19R1', 'C17R1', 'C15R1', 'C13R1', 'B11R1', 'B8R1', 'A8R1', 'A10R1', 'C12R1',
                             'C14R1', 'C16R1', 'C18R1', 'C20R1', 'C22R1', 'C24R1', 'C26R1', 'C28R1', 'C30R1', 'C32R1',
                             'C34R1', 'C33L2', 'C31L2', 'C29L2', 'C27L2', 'C25L2', 'C23L2', 'C21L2', 'C19L2', 'C17L2',
                             'C15L2', 'C13L2', 'B11L2', 'B8L2']

        self.assertEqual(wildcardNameExp, wildcardNameAct)
        wildcardValuesExp.sort()
        wildcardValuesAct.sort()
        self.assertEqual(wildcardValuesExp, wildcardValuesAct)

    def test_getWildcardForCircuitSystemDbSignal_RB_RBA12_VF_CALS_U_EARTH(self):
        # arrange
        circuit = 'RB'
        circuitName = 'RB.A12'
        system = 'VF'
        db = 'CALS'
        signal = 'U_EARTH'

        # act
        wildcardAct = SignalMetadata.getWildcardForCircuitSystemDbSignal(circuit, circuitName, system, db, signal)
        wildcardNameAct = wildcardAct[0]
        wildcardValuesAct = wildcardAct[1]

        # assert
        wildcardNameExp = 'Voltage feeler'
        wildcardValuesExp = ['A8L2', 'A10L2', 'C12L2', 'C14L2', 'C16L2', 'C18L2', 'C20L2', 'C22L2', 'C24L2', 'C26L2',
                             'C28L2', 'C30L2', 'C32L2', 'C34L2', 'A33R1', 'C31R1', 'C29R1', 'C27R1', 'C25R1', 'C23R1',
                             'C21R1', 'C19R1', 'C17R1', 'C15R1', 'C13R1', 'B11R1', 'B8R1', 'A8R1', 'A10R1', 'C12R1',
                             'C14R1', 'C16R1', 'C18R1', 'C20R1', 'C22R1', 'C24R1', 'C26R1', 'C28R1', 'C30R1', 'C32R1',
                             'C34R1', 'C33L2', 'C31L2', 'C29L2', 'C27L2', 'C25L2', 'C23L2', 'C21L2', 'C19L2', 'C17L2',
                             'C15L2', 'C13L2', 'B11L2', 'B8L2']

        self.assertEqual(wildcardNameExp, wildcardNameAct)
        wildcardValuesExp.sort()
        wildcardValuesAct.sort()
        self.assertEqual(wildcardValuesExp, wildcardValuesAct)

    def test_getWildcardForCircuitSystemDbSignal_RQ_RQDA12_VF_CALS_U_EARTH(self):
        # arrange
        circuit = 'RQ'
        circuitName = 'RQD.A12'
        system = 'VF'
        db = 'CALS'
        signal = 'U_EARTH'

        # act
        wildcardAct = SignalMetadata.getWildcardForCircuitSystemDbSignal(circuit, circuitName, system, db, signal)

        # assert
        wildcardExp = []
        self.assertEqual(wildcardExp, wildcardAct)

    def test_getWildcardForCircuitSystemDbSignal_RB_RBA12_BUSBAR_CALS_U_RES(self):
        # arrange
        circuit = 'RB'
        circuitName = 'RB.A12'
        system = 'BUSBAR'
        db = 'CALS'
        signal = 'U_RES'

        # act
        wildcardAct = SignalMetadata.getWildcardForCircuitSystemDbSignal(circuit, circuitName, system, db, signal)
        wildcardNameAct = wildcardAct[0]
        wildcardValuesAct = wildcardAct[1]

        # assert
        wildcardNameExp = 'Busbar'
        wildcardValuesExp = ['DCBB.8L2.R', 'DCBB.9L2.R', 'DCBB.10L2.R', 'DCBB.11L2.R', 'DCBB.A12L2.R', 'DCBB.B12L2.R',
                             'DCBB.13L2.R', 'DCBB.A14L2.R', 'DCBB.B14L2.R', 'DCBB.15L2.R', 'DCBB.A16L2.R',
                             'DCBB.B16L2.R', 'DCBB.17L2.R', 'DCBB.A18L2.R', 'DCBB.B18L2.R', 'DCBB.19L2.R',
                             'DCBB.A20L2.R', 'DCBB.B20L2.R', 'DCBB.21L2.R', 'DCBB.A22L2.R', 'DCBB.B22L2.R',
                             'DCBB.23L2.R', 'DCBB.A24L2.R', 'DCBB.B24L2.R', 'DCBB.25L2.R', 'DCBB.A26L2.R',
                             'DCBB.B26L2.R', 'DCBB.27L2.R', 'DCBB.A28L2.R', 'DCBB.B28L2.R', 'DCBB.29L2.R',
                             'DCBB.A30L2.R', 'DCBB.B30L2.R', 'DCBB.31L2.R', 'DCBB.A32L2.R', 'DCBB.B32L2.R',
                             'DCBB.33L2.R', 'DCBB.A34L2.R', 'DCBB.B34L2.R', 'DCBB.34R1.R', 'DCBB.B33R1.R',
                             'DCBB.A33R1.R', 'DCBB.32R1.R', 'DCBB.B31R1.R', 'DCBB.A31R1.R', 'DCBB.30R1.R',
                             'DCBB.B29R1.R', 'DCBB.A29R1.R', 'DCBB.28R1.R', 'DCBB.B27R1.R', 'DCBB.A27R1.R',
                             'DCBB.26R1.R', 'DCBB.B25R1.R', 'DCBB.A25R1.R', 'DCBB.24R1.R', 'DCBB.B23R1.R',
                             'DCBB.A23R1.R', 'DCBB.22R1.R', 'DCBB.B21R1.R', 'DCBB.A21R1.R', 'DCBB.20R1.R',
                             'DCBB.B19R1.R', 'DCBB.A19R1.R', 'DCBB.18R1.R', 'DCBB.B17R1.R', 'DCBB.A17R1.R',
                             'DCBB.16R1.R', 'DCBB.B15R1.R', 'DCBB.A15R1.R', 'DCBB.14R1.R', 'DCBB.B13R1.R',
                             'DCBB.A13R1.R', 'DCBB.12R1.R', 'DCBB.11R1.R', 'DCBB.10R1.R', 'DCBB.9R1.R', 'DCBB.8R1.R',
                             'DCBD.7R1.R', 'DCBQ.7R1.L', 'DCBQ.8R1.L', 'DCBQ.9R1.L', 'DCBQ.10R1.L', 'DCBQ.11R1.L',
                             'DCBB.12R1.L', 'DCBB.A13R1.L', 'DCBQ.13R1.L', 'DCBB.14R1.L', 'DCBB.A15R1.L', 'DCBQ.15R1.L',
                             'DCBB.16R1.L', 'DCBB.A17R1.L', 'DCBQ.17R1.L', 'DCBB.18R1.L', 'DCBB.A19R1.L', 'DCBQ.19R1.L',
                             'DCBB.20R1.L', 'DCBB.A21R1.L', 'DCBQ.21R1.L', 'DCBB.22R1.L', 'DCBB.A23R1.L', 'DCBQ.23R1.L',
                             'DCBB.24R1.L', 'DCBB.A25R1.L', 'DCBQ.25R1.L', 'DCBB.26R1.L', 'DCBB.A27R1.L', 'DCBQ.27R1.L',
                             'DCBB.28R1.L', 'DCBB.A29R1.L', 'DCBQ.29R1.L', 'DCBB.30R1.L', 'DCBB.A31R1.L', 'DCBQ.31R1.L',
                             'DCBB.32R1.L', 'DCBB.A33R1.L', 'DCBQ.33R1.L', 'DCBB.34R1.L', 'DCBB.B34L2.L', 'DCBQ.33L2.L',
                             'DCBB.33L2.L', 'DCBB.B32L2.L', 'DCBQ.31L2.L', 'DCBB.31L2.L', 'DCBB.B30L2.L', 'DCBQ.29L2.L',
                             'DCBB.29L2.L', 'DCBB.B28L2.L', 'DCBQ.27L2.L', 'DCBB.27L2.L', 'DCBB.A26L2.L', 'DCBQ.25L2.L',
                             'DCBB.25L2.L', 'DCBB.B24L2.L', 'DCBQ.23L2.L', 'DCBB.23L2.L', 'DCBB.B22L2.L', 'DCBQ.21L2.L',
                             'DCBB.21L2.L', 'DCBB.B20L2.L', 'DCBQ.19L2.L', 'DCBB.19L2.L', 'DCBB.B18L2.L', 'DCBQ.17L2.L',
                             'DCBB.17L2.L', 'DCBB.B16L2.L', 'DCBQ.15L2.L', 'DCBB.15L2.L', 'DCBB.B14L2.L', 'DCBQ.13L2.L',
                             'DCBB.13L2.L', 'DCBB.A12L2.L', 'DCBQ.11L2.L', 'DCBQ.10L2.L', 'DCBQ.9L2.L', 'DCBQ.8L2.L',
                             'DCBD.7L2.L']

        self.assertEqual(wildcardNameExp, wildcardNameAct)
        wildcardValuesExp.sort()
        wildcardValuesAct.sort()
        self.assertEqual(wildcardValuesExp, wildcardValuesAct)

    def test_getWildcardForCircuitSystemDbSignal_RQ_RQDA12_VF_CALS_U_EARTH(self):
        # arrange
        circuit = 'RQ'
        circuitName = 'RQD.A12'
        system = 'VF'
        db = 'CALS'
        signal = 'U_EARTH'

        # act
        wildcardAct = SignalMetadata.getWildcardForCircuitSystemDbSignal(circuit, circuitName, system, db, signal)

        # assert
        wildcardExp = []
        self.assertEqual(wildcardExp, wildcardAct)

    def test_getWildcardForCircuitSystemDbSignal_RB_RBA12_BUSBAR_PM_U_RES(self):
        # arrange
        circuit = 'RB'
        circuitName = 'RB.A12'
        system = 'BUSBAR'
        db = 'PM'
        signal = 'U_RES'

        # act
        wildcardAct = SignalMetadata.getWildcardForCircuitSystemDbSignal(circuit, circuitName, system, db, signal)
        wildcardNameAct = wildcardAct[0]
        wildcardValuesAct = wildcardAct[1]

        # assert
        wildcardNameExp = 'Busbar'
        wildcardValuesExp = ['DCBB.8L2.R', 'DCBB.9L2.R', 'DCBB.10L2.R', 'DCBB.11L2.R', 'DCBB.A12L2.R', 'DCBB.B12L2.R',
                             'DCBB.13L2.R', 'DCBB.A14L2.R', 'DCBB.B14L2.R', 'DCBB.15L2.R', 'DCBB.A16L2.R',
                             'DCBB.B16L2.R', 'DCBB.17L2.R', 'DCBB.A18L2.R', 'DCBB.B18L2.R', 'DCBB.19L2.R',
                             'DCBB.A20L2.R', 'DCBB.B20L2.R', 'DCBB.21L2.R', 'DCBB.A22L2.R', 'DCBB.B22L2.R',
                             'DCBB.23L2.R', 'DCBB.A24L2.R', 'DCBB.B24L2.R', 'DCBB.25L2.R', 'DCBB.A26L2.R',
                             'DCBB.B26L2.R', 'DCBB.27L2.R', 'DCBB.A28L2.R', 'DCBB.B28L2.R', 'DCBB.29L2.R',
                             'DCBB.A30L2.R', 'DCBB.B30L2.R', 'DCBB.31L2.R', 'DCBB.A32L2.R', 'DCBB.B32L2.R',
                             'DCBB.33L2.R', 'DCBB.A34L2.R', 'DCBB.B34L2.R', 'DCBB.34R1.R', 'DCBB.B33R1.R',
                             'DCBB.A33R1.R', 'DCBB.32R1.R', 'DCBB.B31R1.R', 'DCBB.A31R1.R', 'DCBB.30R1.R',
                             'DCBB.B29R1.R', 'DCBB.A29R1.R', 'DCBB.28R1.R', 'DCBB.B27R1.R', 'DCBB.A27R1.R',
                             'DCBB.26R1.R', 'DCBB.B25R1.R', 'DCBB.A25R1.R', 'DCBB.24R1.R', 'DCBB.B23R1.R',
                             'DCBB.A23R1.R', 'DCBB.22R1.R', 'DCBB.B21R1.R', 'DCBB.A21R1.R', 'DCBB.20R1.R',
                             'DCBB.B19R1.R', 'DCBB.A19R1.R', 'DCBB.18R1.R', 'DCBB.B17R1.R', 'DCBB.A17R1.R',
                             'DCBB.16R1.R', 'DCBB.B15R1.R', 'DCBB.A15R1.R', 'DCBB.14R1.R', 'DCBB.B13R1.R',
                             'DCBB.A13R1.R', 'DCBB.12R1.R', 'DCBB.11R1.R', 'DCBB.10R1.R', 'DCBB.9R1.R', 'DCBB.8R1.R',
                             'DCBD.7R1.R', 'DCBQ.7R1.L', 'DCBQ.8R1.L', 'DCBQ.9R1.L', 'DCBQ.10R1.L', 'DCBQ.11R1.L',
                             'DCBB.12R1.L', 'DCBB.A13R1.L', 'DCBQ.13R1.L', 'DCBB.14R1.L', 'DCBB.A15R1.L', 'DCBQ.15R1.L',
                             'DCBB.16R1.L', 'DCBB.A17R1.L', 'DCBQ.17R1.L', 'DCBB.18R1.L', 'DCBB.A19R1.L', 'DCBQ.19R1.L',
                             'DCBB.20R1.L', 'DCBB.A21R1.L', 'DCBQ.21R1.L', 'DCBB.22R1.L', 'DCBB.A23R1.L', 'DCBQ.23R1.L',
                             'DCBB.24R1.L', 'DCBB.A25R1.L', 'DCBQ.25R1.L', 'DCBB.26R1.L', 'DCBB.A27R1.L', 'DCBQ.27R1.L',
                             'DCBB.28R1.L', 'DCBB.A29R1.L', 'DCBQ.29R1.L', 'DCBB.30R1.L', 'DCBB.A31R1.L', 'DCBQ.31R1.L',
                             'DCBB.32R1.L', 'DCBB.A33R1.L', 'DCBQ.33R1.L', 'DCBB.34R1.L', 'DCBB.B34L2.L', 'DCBQ.33L2.L',
                             'DCBB.33L2.L', 'DCBB.B32L2.L', 'DCBQ.31L2.L', 'DCBB.31L2.L', 'DCBB.B30L2.L', 'DCBQ.29L2.L',
                             'DCBB.29L2.L', 'DCBB.B28L2.L', 'DCBQ.27L2.L', 'DCBB.27L2.L', 'DCBB.A26L2.L', 'DCBQ.25L2.L',
                             'DCBB.25L2.L', 'DCBB.B24L2.L', 'DCBQ.23L2.L', 'DCBB.23L2.L', 'DCBB.B22L2.L', 'DCBQ.21L2.L',
                             'DCBB.21L2.L', 'DCBB.B20L2.L', 'DCBQ.19L2.L', 'DCBB.19L2.L', 'DCBB.B18L2.L', 'DCBQ.17L2.L',
                             'DCBB.17L2.L', 'DCBB.B16L2.L', 'DCBQ.15L2.L', 'DCBB.15L2.L', 'DCBB.B14L2.L', 'DCBQ.13L2.L',
                             'DCBB.13L2.L', 'DCBB.A12L2.L', 'DCBQ.11L2.L', 'DCBQ.10L2.L', 'DCBQ.9L2.L', 'DCBQ.8L2.L',
                             'DCBD.7L2.L']

        self.assertEqual(wildcardNameExp, wildcardNameAct)
        wildcardValuesExp.sort()
        wildcardValuesAct.sort()
        self.assertEqual(wildcardValuesExp, wildcardValuesAct)

    def test_getWildcardForCircuitSystemDbSignal_RB_RBA12_DIODE_PM_U_REF_N1(self):
        # arrange
        circuit = 'RB'
        circuitName = 'RB.A12'
        system = 'DIODE_RB'
        db = 'PM'
        signal = 'U_REF_N1'

        # act
        wildcardAct = SignalMetadata.getWildcardForCircuitSystemDbSignal(circuit, circuitName, system, db, signal)
        wildcardNameAct = wildcardAct[0]
        wildcardValuesAct = wildcardAct[1]

        # assert
        wildcardNameExp = 'Crate'
        wildcardValuesExp = ['B8L2', 'B10L2', 'B12L2', 'B14L2', 'B16L2', 'B18L2', 'B20L2', 'B22L2', 'B24L2', 'B26L2',
                             'B28L2', 'B30L2', 'B32L2', 'B34L2', 'B33R1', 'B31R1', 'B29R1', 'B27R1', 'B25R1', 'B23R1',
                             'B21R1', 'B19R1', 'B17R1', 'B15R1', 'B13R1', 'B11R1', 'B9R1', 'B8R1', 'B10R1', 'B12R1',
                             'B14R1', 'B16R1', 'B18R1', 'B20R1', 'B22R1', 'B24R1', 'B26R1', 'B28R1', 'B30R1', 'B32R1',
                             'B34R1', 'B33L2', 'B31L2', 'B29L2', 'B27L2', 'B25L2', 'B23L2', 'B21L2', 'B19L2', 'B17L2',
                             'B15L2', 'B13L2', 'B11L2', 'B9L2']

        self.assertEqual(wildcardNameExp, wildcardNameAct)
        wildcardValuesExp.sort()
        wildcardValuesAct.sort()
        self.assertEqual(wildcardValuesExp, wildcardValuesAct)

    def test_getWildcardForCircuitSystemDbSignal_RB_RBA12_DIODE_PM_U_DIODE_RB(self):
        # arrange
        circuit = 'RB'
        circuitName = 'RB.A12'
        system = 'DIODE_RB'
        db = 'PM'
        signal = 'U_DIODE_RB'

        # act
        wildcardAct = SignalMetadata.getWildcardForCircuitSystemDbSignal(circuit, circuitName, system, db, signal)
        wildcardNameAct = wildcardAct[0]
        wildcardValuesAct = wildcardAct[1]

        # assert
        wildcardNameExp = 'Magnet'
        wildcardValuesExp = ['MB.A8R1', 'MB.B8R1', 'MB.A9R1', 'MB.B9R1', 'MB.A10R1', 'MB.B10R1', 'MB.A11R1', 'MB.B11R1', 'MB.A12R1', 'MB.B12R1',
                             'MB.C12R1', 'MB.A13R1', 'MB.B13R1', 'MB.C13R1', 'MB.A14R1', 'MB.B14R1', 'MB.C14R1', 'MB.A15R1', 'MB.B15R1', 'MB.C15R1',
                             'MB.A16R1', 'MB.B16R1', 'MB.C16R1', 'MB.A17R1', 'MB.B17R1', 'MB.C17R1', 'MB.A18R1', 'MB.B18R1', 'MB.C18R1', 'MB.A19R1',
                             'MB.B19R1', 'MB.C19R1', 'MB.A20R1', 'MB.B20R1', 'MB.C20R1', 'MB.A21R1', 'MB.B21R1', 'MB.C21R1', 'MB.A22R1', 'MB.B22R1',
                             'MB.C22R1', 'MB.A23R1', 'MB.B23R1', 'MB.C23R1', 'MB.A24R1', 'MB.B24R1', 'MB.C24R1', 'MB.A25R1', 'MB.B25R1', 'MB.C25R1',
                             'MB.A26R1', 'MB.B26R1', 'MB.C26R1', 'MB.A27R1', 'MB.B27R1', 'MB.C27R1', 'MB.A28R1', 'MB.B28R1', 'MB.C28R1', 'MB.A29R1',
                             'MB.B29R1', 'MB.C29R1', 'MB.A30R1', 'MB.B30R1', 'MB.C30R1', 'MB.A31R1', 'MB.B31R1', 'MB.C31R1', 'MB.A32R1', 'MB.B32R1',
                             'MB.C32R1', 'MB.A33R1', 'MB.B33R1', 'MB.C33R1', 'MB.A34R1', 'MB.B34R1', 'MB.C34R1', 'MB.C34L2', 'MB.B34L2', 'MB.A34L2',
                             'MB.C33L2', 'MB.B33L2', 'MB.A33L2', 'MB.C32L2', 'MB.B32L2', 'MB.A32L2', 'MB.C31L2', 'MB.B31L2', 'MB.A31L2', 'MB.C30L2',
                             'MB.B30L2', 'MB.A30L2', 'MB.C29L2', 'MB.B29L2', 'MB.A29L2', 'MB.C28L2', 'MB.B28L2', 'MB.A28L2', 'MB.C27L2', 'MB.B27L2',
                             'MB.A27L2', 'MB.C26L2', 'MB.B26L2', 'MB.A26L2', 'MB.C25L2', 'MB.B25L2', 'MB.A25L2', 'MB.C24L2', 'MB.B24L2', 'MB.A24L2',
                             'MB.C23L2', 'MB.B23L2', 'MB.A23L2', 'MB.C22L2', 'MB.B22L2', 'MB.A22L2', 'MB.C21L2', 'MB.B21L2', 'MB.A21L2', 'MB.C20L2',
                             'MB.B20L2', 'MB.A20L2', 'MB.C19L2', 'MB.B19L2', 'MB.A19L2', 'MB.C18L2', 'MB.B18L2', 'MB.A18L2', 'MB.C17L2', 'MB.B17L2',
                             'MB.A17L2', 'MB.C16L2', 'MB.B16L2', 'MB.A16L2', 'MB.C15L2', 'MB.B15L2', 'MB.A15L2', 'MB.C14L2', 'MB.B14L2', 'MB.A14L2',
                             'MB.C13L2', 'MB.B13L2', 'MB.A13L2', 'MB.C12L2', 'MB.B12L2', 'MB.A12L2', 'MB.B11L2', 'MB.A11L2', 'MB.B10L2', 'MB.A10L2',
                             'MB.B9L2', 'MB.A9L2', 'MB.B8L2', 'MB.A8L2']

        self.assertEqual(wildcardNameExp, wildcardNameAct)
        wildcardValuesExp.sort()
        wildcardValuesAct.sort()
        self.assertEqual(wildcardValuesExp, wildcardValuesAct)

    def test_updatePMSourceWithWildcard_RB_QH_U_HDS(self):
        # arrange
        sourceIn = '*'
        circuit = 'RB'
        system = 'QH'
        signal = 'U_HDS'
        wildcard = ['A8R1', 'B8R1', 'A9R1']

        # act
        sourceAct = SignalMetadata.updatePMSourceWithWildcard(sourceIn, circuit, system, signal, wildcard)

        # assert
        sourceExp = wildcard
        sourceExp.sort()
        sourceAct.sort()
        self.assertEqual(sourceExp, sourceAct)

    def test_updatePMSourceWithWildcard_RB_QDS_U_HDS(self):
        # arrange
        sourceIn = '*'
        circuit = 'RB'
        system = 'QDS'
        signal = 'U_HDS'
        wildcard = 'A8R1'

        # act
        sourceAct = SignalMetadata.updatePMSourceWithWildcard(sourceIn, circuit, system, signal, wildcard)

        # assert
        sourceExp = [wildcard]
        self.assertEqual(sourceExp, sourceAct)

    def test_updatePMSourceWithWildcard_RB_BUSBAR_U_RES(self):
        # arrange
        sourceIn = '*'
        circuit = 'RB'
        system = 'BUSBAR'
        signal = 'U_RES'
        wildcard = ['DCBB.8L2.R', 'DCBB.9L2.R', 'DCBB.10L2.R']

        # act
        sourceAct = SignalMetadata.updatePMSourceWithWildcard(sourceIn, circuit, system, signal, wildcard)

        # assert
        sourceExp = ['B10L2', 'B8L2', 'B8L2']
        sourceExp.sort()
        sourceAct.sort()
        self.assertEqual(sourceExp, sourceAct)

    def test_updatePMSourceWithWildcard_RB_DIODE_U_REF_N1(self):
        # arrange
        sourceIn = '*'
        circuit = 'RB'
        system = 'DIODE_RB'
        signal = 'U_REF_N1'
        wildcard = ['MB.A8R1', 'MB.B8R1', 'MB.A9R1']

        # act
        sourceAct = SignalMetadata.updatePMSourceWithWildcard(sourceIn, circuit, system, signal, wildcard)

        # assert
        sourceExp = ['MB.A8R1', 'MB.B8R1', 'MB.A9R1']
        sourceExp.sort()
        sourceAct.sort()
        self.assertEqual(sourceExp, sourceAct)

    def test_updatePMSourceWithWildcard_RB_DIODE_U_DIODE_RB(self):
        # arrange
        sourceIn = '*'
        circuit = 'RB'
        system = 'DIODE_RB'
        signal = 'U_DIODE_RB'
        wildcard = ['MB.A8R1', 'MB.B8R1', 'MB.A9R1']

        # act
        sourceAct = SignalMetadata.updatePMSourceWithWildcard(sourceIn, circuit, system, signal, wildcard)

        # assert
        sourceExp = ['B10R1', 'B8R1', 'B9R1']
        sourceExp.sort()
        sourceAct.sort()
        self.assertEqual(sourceExp, sourceAct)

    def test_updatePMSourceWithWildcard_RB_VF_U_EARTH(self):
        # arrange
        sourceIn = "%QPS_CRATE%"
        circuit = 'RB'
        system = 'VF'
        signal = 'U_EARTH'
        wildcard = ['MB.A8R1', 'MB.B8R1', 'MB.A9R1']

        # act
        sourceAct = SignalMetadata.updatePMSourceWithWildcard(sourceIn, circuit, system, signal, wildcard)

        # assert
        sourceExp = ['B10R1', 'B8R1', 'B9R1']
        sourceExp.sort()
        sourceAct.sort()
        self.assertEqual(sourceExp, sourceAct)

    # UPDATE PM SIGNAL NAMES WITH WILDCARD
    def test_updatePMSignalNamesWithWildcard_RB_QH_U_HDS(self):
        # arrange
        signalNamesIn = ['U_HDS_1', 'U_HDS_2', 'U_HDS_3', 'U_HDS_4']
        system = 'QH'
        signal = 'U_HDS'
        wildcard = ['A8R1', 'B8R1', 'A9R1']

        # act
        signalNamesAct = SignalMetadata.updatePMSignalNamesWithWildcard(signalNamesIn, system, signal, wildcard)

        # assert
        signalNamesExp = ['U_HDS_1', 'U_HDS_2', 'U_HDS_3', 'U_HDS_4']
        signalNamesExp.sort()
        signalNamesAct.sort()
        self.assertEqual(signalNamesExp, signalNamesAct)

    def test_updatePMSignalNamesWithWildcard_RB_QDS_U_HDS(self):
        # arrange
        signalNamesIn = ["%CELL%:U_QS0", "%CELL%:U_1", "%CELL%:U_2"]
        system = 'QDS'
        signal = 'U_HDS'
        wildcard = ['A8R1', 'B8R1', 'A9R1']

        # act
        signalNamesAct = SignalMetadata.updatePMSignalNamesWithWildcard(signalNamesIn, system, signal, wildcard)

        # assert
        signalNamesExp = ['A8R1:U_QS0', 'A8R1:U_1', 'A8R1:U_2', 'B8R1:U_QS0', 'B8R1:U_1', 'B8R1:U_2', 'A9R1:U_QS0', 'A9R1:U_1', 'A9R1:U_2']
        signalNamesExp.sort()
        signalNamesAct.sort()
        self.assertEqual(signalNamesExp, signalNamesAct)

    def test_updatePMSignalNamesWithWildcard_RB_BUSBAR_U_RES(self):
        # arrange
        signalNamesIn = ["%BUSBAR%:U_MAG", "%BUSBAR%:U_RES", "%BUSBAR%:ST_BUSBAR_OK", "%BUSBAR%:ST_PWR_PERM"]
        system = 'BUSBAR'
        signal = 'U_RES'
        wildcard = ['DCBB.8L2.R', 'DCBB.9L2.R', 'DCBB.10L2.R']

        # act
        signalNamesAct = SignalMetadata.updatePMSignalNamesWithWildcard(signalNamesIn, system, signal, wildcard)

        # assert
        signalNamesExp = ['DCBB.8L2.R:U_MAG', 'DCBB.8L2.R:U_RES', 'DCBB.8L2.R:ST_BUSBAR_OK', 'DCBB.8L2.R:ST_PWR_PERM',
                          'DCBB.9L2.R:U_MAG', 'DCBB.9L2.R:U_RES', 'DCBB.9L2.R:ST_BUSBAR_OK', 'DCBB.9L2.R:ST_PWR_PERM',
                          'DCBB.10L2.R:U_MAG', 'DCBB.10L2.R:U_RES', 'DCBB.10L2.R:ST_BUSBAR_OK', 'DCBB.10L2.R:ST_PWR_PERM']
        signalNamesAct.sort()
        signalNamesExp.sort()
        self.assertEqual(signalNamesAct, signalNamesExp)

    def test_updatePMSignalNamesWithWildcard_RB_DIODE_U_REF_N1(self):
        # arrange
        signalNamesIn = ["DQQDS.%QPS_CRATE%.RB.A12:U_REF_N1"]
        system = 'DIODE'
        signal = 'U_REF_N1'
        wildcard = ['B8L2', 'B10L2', 'B12L2']

        # act
        signalNamesAct = SignalMetadata.updatePMSignalNamesWithWildcard(signalNamesIn, system, signal, wildcard)

        # assert
        signalNamesExp = ['DQQDS.B8L2.RB.A12:U_REF_N1', 'DQQDS.B10L2.RB.A12:U_REF_N1', 'DQQDS.B12L2.RB.A12:U_REF_N1']
        signalNamesExp.sort()
        signalNamesAct.sort()
        self.assertEqual(signalNamesExp, signalNamesAct)

    def test_updatePMSignalNamesWithWildcard_RB_DIODE_U_DIODE_RB(self):
        # arrange
        signalNamesIn = "%MAGNET%:ST_MAGNET_OK_SQ_RB"
        system = 'DIODE'
        signal = 'U_DIODE_RB'
        wildcard = ['MB.A8R1', 'MB.B8R1', 'MB.A9R1']

        # act
        signalNamesAct = SignalMetadata.updatePMSignalNamesWithWildcard(signalNamesIn, system, signal, wildcard)

        # assert
        signalNamesExp = ['MB.A8R1:ST_MAGNET_OK_SQ_RB', 'MB.B8R1:ST_MAGNET_OK_SQ_RB', 'MB.A9R1:ST_MAGNET_OK_SQ_RB']
        signalNamesExp.sort()
        signalNamesAct.sort()
        self.assertEqual(signalNamesExp, signalNamesAct)

    def test_updatePMSignalNamesWithWildcard_RB_VF_U_EARTH(self):
        # arrange
        signalNamesIn = "%MAGNET%:U_EARTH_RB"
        system = 'VF'
        signal = 'U_EARTH_RB'
        wildcard = ['MB.A8L2', 'MB.A10L2', 'MB.C12L2', 'MB.C14L2']

        # act
        signalNamesAct = SignalMetadata.updatePMSignalNamesWithWildcard(signalNamesIn, system, signal, wildcard)

        # assert
        sourceExp = ['MB.A8L2:U_EARTH_RB', 'MB.A10L2:U_EARTH_RB', 'MB.C12L2:U_EARTH_RB', 'MB.C14L2:U_EARTH_RB']
        sourceExp.sort()
        signalNamesAct.sort()
        self.assertEqual(sourceExp, signalNamesAct)

    def test_updatePMSignalNamesWithWildcard_RB_PC_I_MEAS(self):
        # arrange
        signalNamesIn = "STATUS.I_REF"
        system = 'PC'
        signal = 'I_REF'
        wildcard = ''

        # act
        signalNamesAct = SignalMetadata.updatePMSignalNamesWithWildcard(signalNamesIn, system, signal, wildcard)

        # assert
        sourceExp = ["STATUS.I_REF"]
        sourceExp.sort()
        signalNamesAct.sort()
        self.assertEqual(sourceExp, signalNamesAct)

        # UPDATE CALS SIGNAL NAMES WITH WILDCARD

    def test_updateCALSSignalNamesWithWildcard_RB_QH_U_HDS(self):
        # arrange
        signalNamesIn = ["%MAGNET%:U_HDS_1", "%MAGNET%:U_HDS_2", "%MAGNET%:U_HDS_3", "%MAGNET%:U_HDS_4"]
        system = 'QH'
        wildcard = ['A8R1', 'B8R1', 'A9R1']

        # act
        signalNamesAct = SignalMetadata.updateCALSSignalNamesWithWildcard(signalNamesIn, system, wildcard)

        # assert
        signalNamesExp = ['A8R1:U_HDS_1', 'A8R1:U_HDS_2', 'A8R1:U_HDS_3', 'A8R1:U_HDS_4', 'B8R1:U_HDS_1',
                          'B8R1:U_HDS_2', 'B8R1:U_HDS_3', 'B8R1:U_HDS_4', 'A9R1:U_HDS_1', 'A9R1:U_HDS_2',
                          'A9R1:U_HDS_3', 'A9R1:U_HDS_4']

        signalNamesExp.sort()
        signalNamesAct.sort()
        self.assertEqual(signalNamesExp, signalNamesAct)

    def test_updateVSLDSignalNamesWithWildcard_RB_QDS_U_HDS(self):
        # arrange
        signalNamesIn = ["%MAGNET%:U_QS0", "%MAGNET%:U_1", "%MAGNET%:U_2"]
        system = 'QDS'
        wildcard = ['MB.A8R1', 'MB.B8R1', 'MB.A9R1']

        # act
        signalNamesAct = SignalMetadata.updateCALSSignalNamesWithWildcard(signalNamesIn, system, wildcard)

        # assert
        signalNamesExp = ['MB.A8R1:U_QS0', 'MB.A8R1:U_1', 'MB.A8R1:U_2', 'MB.B8R1:U_QS0', 'MB.B8R1:U_1', 'MB.B8R1:U_2',
                          'MB.A9R1:U_QS0', 'MB.A9R1:U_1', 'MB.A9R1:U_2']
        signalNamesExp.sort()
        signalNamesAct.sort()
        self.assertEqual(signalNamesExp, signalNamesAct)

    def test_updateCALSSignalNamesWithWildcard_RB_BUSBAR_U_RES(self):
        # arrange
        signalNamesIn = ["%BUSBAR%:U_MAG", "%BUSBAR%:U_RES", "%BUSBAR%:ST_BUSBAR_OK", "%BUSBAR%:ST_PWR_PERM"]
        system = 'BUSBAR'
        wildcard = ['DCBB.8L2.R', 'DCBB.9L2.R', 'DCBB.10L2.R']

        # act
        signalNamesAct = SignalMetadata.updateCALSSignalNamesWithWildcard(signalNamesIn, system, wildcard)

        # assert
        signalNamesExp = ['DCBB.8L2.R:U_MAG', 'DCBB.8L2.R:U_RES', 'DCBB.8L2.R:ST_BUSBAR_OK',
                          'DCBB.8L2.R:ST_PWR_PERM',
                          'DCBB.9L2.R:U_MAG', 'DCBB.9L2.R:U_RES', 'DCBB.9L2.R:ST_BUSBAR_OK',
                          'DCBB.9L2.R:ST_PWR_PERM',
                          'DCBB.10L2.R:U_MAG', 'DCBB.10L2.R:U_RES', 'DCBB.10L2.R:ST_BUSBAR_OK',
                          'DCBB.10L2.R:ST_PWR_PERM']
        signalNamesAct.sort()
        signalNamesExp.sort()
        self.assertEqual(signalNamesAct, signalNamesExp)

    def test_updateCALSSignalNamesWithWildcard_RB_DIODE_U_DIODE_RB(self):
        # arrange
        signalNamesIn = "%MAGNET%:ST_MAGNET_OK_SQ_RB"
        system = 'DIODE'
        wildcard = ['MB.A8R1', 'MB.B8R1', 'MB.A9R1']

        # act
        signalNamesAct = SignalMetadata.updateCALSSignalNamesWithWildcard(signalNamesIn, system, wildcard)

        # assert
        signalNamesExp = ['MB.A8R1:ST_MAGNET_OK_SQ_RB', 'MB.B8R1:ST_MAGNET_OK_SQ_RB', 'MB.A9R1:ST_MAGNET_OK_SQ_RB']
        signalNamesExp.sort()
        signalNamesAct.sort()
        self.assertEqual(signalNamesExp, signalNamesAct)

    def test_updateCALSSignalNamesWithWildcard_RB_VF_U_EARTH(self):
        # arrange
        signalNamesIn = "%MAGNET%:U_EARTH_RB"
        system = 'VF'
        wildcard = ['MB.A8L2', 'MB.A10L2', 'MB.C12L2', 'MB.C14L2']

        # act
        signalNamesAct = SignalMetadata.updateCALSSignalNamesWithWildcard(signalNamesIn, system, wildcard)

        # assert
        sourceExp = ['MB.A8L2:U_EARTH_RB', 'MB.A10L2:U_EARTH_RB', 'MB.C12L2:U_EARTH_RB', 'MB.C14L2:U_EARTH_RB']
        sourceExp.sort()
        signalNamesAct.sort()
        self.assertEqual(sourceExp, signalNamesAct)

    def test_updateCALSSignalNamesWithWildcard_RB_PC_I_MEAS(self):
        # arrange
        signalNamesIn = "STATUS.I_REF"
        system = 'PC'
        wildcard = ''

        # act
        signalNamesAct = SignalMetadata.updateCALSSignalNamesWithWildcard(signalNamesIn, system, wildcard)

        # assert
        sourceExp = ["STATUS.I_REF"]
        sourceExp.sort()
        signalNamesAct.sort()
        self.assertEqual(sourceExp, signalNamesAct)

    def test_RB_Metadata_getSectorNamesFromMagnetNames(self):
        """Tests that Metadata returns correct  sector name from magnet names"""

        # arrange
        circuitType = "RB"
        magnet = "B20L5"
        component = "MAGNET"

        # act
        sectorAct = SignalMetadata.getSectorNamesFromComponents(circuitType, component, magnet)

        # assert
        sectorExp = "RB.A45"
        self.assertEqual(sectorExp, sectorAct)

    def test_RB_Metadata_getSectorNamesFromBusbarNames(self):
        """Tests that Metadata returns correct  sector name from busbar names"""

        # arrange
        circuitType = "RB"
        busbar = "DCBA.10L1.L:U_MAG"
        component = "BUSBAR"

        # act
        sectorAct = SignalMetadata.getSectorNamesFromComponents(circuitType, component, busbar)

        # assert
        sectorExp = "RB.A81"
        self.assertEqual(sectorExp, sectorAct)

    def test_RB_Metadata_getSectorNamesFromCurrentName(self):
        """Tests that Metadata returns correct sector name from current name"""

        # arrange
        circuitType = "RB"
        current = "RPTE.UA23.RB.A12:I_MEAS"
        component = "PC"

        # act
        sectorAct = SignalMetadata.getSectorNamesFromComponents(circuitType, component, current)

        # assert
        sectorExp = "RB.A12"
        self.assertEqual(sectorExp, sectorAct)

    def test_RB_Metadata_getSectorNamesFromCurrentNames(self):
        """Tests that Metadata returns correct sector name from current names"""

        # arrange
        circuitType = "RB"
        current = ["RPTE.UA23.RB.A12:I_MEAS", "RPTE.UA27.RB.A23:I_MEAS"]
        component = "PC"

        # act
        sectorAct = SignalMetadata.getSectorNamesFromComponents(circuitType, component, current)

        # assert
        sectorExp = ["RB.A12", "RB.A23"]
        self.assertListEqual(sectorExp, sectorAct)