import unittest

from lhcsmapi.post_mortem.PmDbRequest import PmDbRequest

class TestPmDbRequest(unittest.TestCase):

    def setUp(self):
        self.SYSTEM = "QPS"
        self.CLASS_NAME = "DQAMCNMB_PMHSU"
        self.SOURCE = "B20L5"
        self.SIGNAL_NAME = "I_HDS_4"
        self.TIMESTAMP = 1426220469491000000
        self.DURATION = 86400000000000

    def tearDown(self):
        pass

    # region generateQuery tests
    def test_generateSignalRequest_Qps_Parameters(self):
        queryExpected = "http://pm-api-pro/v2/pmdata/signal?system=QPS&className=DQAMCNMB_PMHSU&source=B20L5&timestampInNanos=1426220469491000000&signal=I_HDS_4"
        queryActual = PmDbRequest.generateRequest("pmdata/signal", False, system = self.SYSTEM, className = self.CLASS_NAME,
                                                  source = self.SOURCE, timestampInNanos = self.TIMESTAMP, signal = self.SIGNAL_NAME)
        self.assertEqual(len(queryExpected), len(queryActual))

    def test_generateEventRequest_Qps_Parameters_Non_Zero_Duration_Header_True(self):
        queryExpected = "http://pm-api-pro/v2/pmevent/header/within/duration?system=QPS&className=DQAMCNMB_PMHSU&source=B20L5&fromTimestampInNanos=1426220469491000000&durationInNanos=86400000000000"
        queryActual = PmDbRequest.generateRequest("pmevent", True, system = self.SYSTEM, className = self.CLASS_NAME, source = self.SOURCE,
                                                  fromTimestampInNanos = self.TIMESTAMP, durationInNanos = self.DURATION)
        self.assertEqual(len(queryExpected), len(queryActual))

    def test_generateEventRequest_Qps_Parameters_Non_Zero_Duration_Header_False(self):
        queryExpected = "http://pm-api-pro/v2/pmevent/within/duration?system=QPS&className=DQAMCNMB_PMHSU&source=B20L5&fromTimestampInNanos=1426220469491000000&durationInNanos=86400000000000"
        queryActual = PmDbRequest.generateRequest("pmevent", False, system = self.SYSTEM, className = self.CLASS_NAME, source = self.SOURCE,
                                                  fromTimestampInNanos = self.TIMESTAMP, durationInNanos = self.DURATION)
        self.assertEqual(len(queryExpected), len(queryActual))

    def test_generateEventRequest_Qps_Parameters_Zero_Duration_Header_True(self):
        queryExpected = "http://pm-api-pro/v2/pmevent/header?system=QPS&className=DQAMCNMB_PMHSU&source=B20L5&timestampInNanos=1426220469491000000"
        queryActual = PmDbRequest.generateRequest("pmevent", True,
                                                  system = self.SYSTEM, className = self.CLASS_NAME, source = self.SOURCE,
                                                  timestampInNanos = self.TIMESTAMP)
        self.assertEqual(len(queryExpected), len(queryActual))

    def test_generateEventRequest_Qps_Parameters_Zero_Duration_Header_False(self):
        """Tests that the default CalsSignal is created if no arguments are passed to the createSignalCALS method"""
        queryExpected = "http://pm-api-pro/v2/pmevent?system=QPS&className=DQAMCNMB_PMHSU&source=B20L5&timestampInNanos=1426220469491000000"
        queryActual = PmDbRequest.generateRequest("pmevent", False, system = self.SYSTEM, className = self.CLASS_NAME, source = self.SOURCE,
                                                  timestampInNanos = self.TIMESTAMP)
        self.assertEqual(len(queryExpected), len(queryActual))

    def test_generateDataRequest_Qps_Parameters_Non_Zero_Duration_Header_True(self):
        queryExpected = "http://pm-api-pro/v2/pmdata/header/within/duration?system=QPS&className=DQAMCNMB_PMHSU&source=B20L5&fromTimestampInNanos=1426220469491000000&durationInNanos=86400000000000"
        queryActual = PmDbRequest.generateRequest("pmdata", True, system = self.SYSTEM, className = self.CLASS_NAME, source = self.SOURCE,
                                                  fromTimestampInNanos = self.TIMESTAMP, durationInNanos = self.DURATION)
        self.assertEqual(len(queryExpected), len(queryActual))

    def test_generateDataRequest_Qps_Parameters_Non_Zero_Duration_Header_False(self):
        queryExpected = "http://pm-api-pro/v2/pmdata/within/duration?system=QPS&className=DQAMCNMB_PMHSU&source=B20L5&fromTimestampInNanos=1426220469491000000&durationInNanos=86400000000000"
        queryActual = PmDbRequest.generateRequest("pmdata", False, system = self.SYSTEM, className = self.CLASS_NAME, source = self.SOURCE,
                                                  fromTimestampInNanos = self.TIMESTAMP, durationInNanos = self.DURATION)
        self.assertEqual(len(queryExpected), len(queryActual))

    def test_generateDataRequest_Qps_Parameters_Zero_Duration_Header_True(self):
        queryExpected = "http://pm-api-pro/v2/pmdata/header?system=QPS&className=DQAMCNMB_PMHSU&source=B20L5&timestampInNanos=1426220469491000000"
        queryActual = PmDbRequest.generateRequest("pmdata", True, system = self.SYSTEM, className = self.CLASS_NAME,
                                                  source = self.SOURCE, timestampInNanos = self.TIMESTAMP)
        self.assertEqual(len(queryExpected), len(queryActual))

    def test_generateDataRequest_Qps_Parameters_Zero_Duration_Header_False(self):
        queryExpected = "http://pm-api-pro/v2/pmdata?system=QPS&className=DQAMCNMB_PMHSU&source=B20L5&timestampInNanos=1426220469491000000"
        queryActual = PmDbRequest.generateRequest("pmdata", False, system = self.SYSTEM, className = self.CLASS_NAME,
                                                  source = self.SOURCE, timestampInNanos = self.TIMESTAMP)
        self.assertEqual(len(queryExpected), len(queryActual))

    # endregion
